// file      : CCF/IDL3/Traversal/Operation.hpp
// author    : Boris Kolpackov <boris@dre.vanderbilt.edu>
// cvs-id    : Operation.hpp,v 1.3 2003/10/14 21:54:18 boris Exp

#ifndef CCF_IDL3_TRAVERSAL_OPERATION_HPP
#define CCF_IDL3_TRAVERSAL_OPERATION_HPP

#include "CCF/IDL2/Traversal/Operation.hpp"
#include "CCF/IDL3/SyntaxTree/Operation.hpp"

namespace CCF
{
  namespace IDL3
  {
    namespace Traversal
    {
      //
      //
      //
      struct ProvidesDecl : IDL2::Traversal::Traverser
      {
        typedef
        SyntaxTree::ProvidesDeclPtr
        NodePtr;

        ProvidesDecl (Dispatcher* type = 0)
            : type_ (type)
        {
          map (typeid (SyntaxTree::ProvidesDecl), this);
        }

        virtual bool
        traverse (SyntaxTree::NodePtr const& n)
        {
          traverse (n->dynamic_type<SyntaxTree::ProvidesDecl> ());
	  return true;
        }

        virtual void
        traverse (NodePtr const&);

        virtual void
        pre (NodePtr const&);

        virtual void
        type (NodePtr const&);

        virtual void
        post (NodePtr const&);

      private:
        Dispatcher* type_;
      };


      //
      //
      //
      struct UsesDecl : IDL2::Traversal::Traverser
      {
        typedef
        SyntaxTree::UsesDeclPtr
        NodePtr;

        UsesDecl (Dispatcher* type = 0)
            : type_ (type)
        {
          map (typeid (SyntaxTree::UsesDecl), this);
        }

        virtual bool
        traverse (SyntaxTree::NodePtr const& n)
        {
          traverse (n->dynamic_type<SyntaxTree::UsesDecl> ());
	  return true;
        }

        virtual void
        traverse (NodePtr const&);

        virtual void
        pre (NodePtr const&);

        virtual void
        type (NodePtr const&);

        virtual void
        post (NodePtr const&);

      private:
        Dispatcher* type_;
      };


      //
      //
      //
      struct PublishesDecl : IDL2::Traversal::Traverser
      {
        typedef
        SyntaxTree::PublishesDeclPtr
        NodePtr;

        PublishesDecl (Dispatcher* type = 0)
            : type_ (type)
        {
          map (typeid (SyntaxTree::PublishesDecl), this);
        }

        virtual bool
        traverse (SyntaxTree::NodePtr const& n)
        {
          traverse (n->dynamic_type<SyntaxTree::PublishesDecl> ());
	  return true;
        }

        virtual void
        traverse (NodePtr const&);

        virtual void
        pre (NodePtr const&);

        virtual void
        type (NodePtr const&);

        virtual void
        post (NodePtr const&);

      private:
        Dispatcher* type_;
      };


      //
      //
      //
      struct EmitsDecl : IDL2::Traversal::Traverser
      {
        typedef
        SyntaxTree::EmitsDeclPtr
        NodePtr;

        EmitsDecl (Dispatcher* type = 0)
            : type_ (type)
        {
          map (typeid (SyntaxTree::EmitsDecl), this);
        }

        virtual bool
        traverse (SyntaxTree::NodePtr const& n)
        {
          traverse (n->dynamic_type<SyntaxTree::EmitsDecl> ());
	  return true;
        }

        virtual void
        traverse (NodePtr const&);

        virtual void
        pre (NodePtr const&);

        virtual void
        type (NodePtr const&);

        virtual void
        post (NodePtr const&);

      private:
        Dispatcher* type_;
      };


      //
      //
      //
      struct ConsumesDecl : IDL2::Traversal::Traverser
      {
        typedef
        SyntaxTree::ConsumesDeclPtr
        NodePtr;

        ConsumesDecl (Dispatcher* type = 0)
            : type_ (type)
        {
          map (typeid (SyntaxTree::ConsumesDecl), this);
        }

        virtual bool
        traverse (SyntaxTree::NodePtr const& n)
        {
          traverse (n->dynamic_type<SyntaxTree::ConsumesDecl> ());
	  return true;
        }

        virtual void
        traverse (NodePtr const&);

        virtual void
        pre (NodePtr const&);

        virtual void
        type (NodePtr const&);

        virtual void
        post (NodePtr const&);

      private:
        Dispatcher* type_;
      };


      //
      //
      //
      struct HomeFactoryDecl : IDL2::Traversal::OperationTraverser
      {
        typedef
        SyntaxTree::HomeFactoryDeclPtr
        NodePtr;

        HomeFactoryDecl (Dispatcher* type = 0)
            : OperationTraverser (type)
        {
          map (typeid (SyntaxTree::HomeFactoryDecl), this);
        }

        virtual bool
        traverse (SyntaxTree::NodePtr const& n)
        {
          traverse (n->dynamic_type<SyntaxTree::HomeFactoryDecl> ());
          return true;
        }

        virtual void
        traverse (NodePtr const&);

        virtual void
        pre (NodePtr const&);

        virtual void
        type (NodePtr const&);

        virtual void
        name (NodePtr const&);

        virtual void
        parameters (NodePtr const&);

        virtual void
        post (NodePtr const&);

      private:
        Dispatcher* type_;
      };
    }
  }
}

#endif  // CCF_IDL3_TRAVERSAL_OPERATION_HPP
